<!-- questions:

1. what is 'traits'?
3. should proto schema be extended by anything which could be found through segment?

 -->


## Event Structure - DRAFT

### Common Attributes
* `deviceId` - device Id
* `messageId` - Client generated unique id for the event
* `eventTs` - event timestamp
* `context` - metadata attributes wrapper (device information, location, os, network etc.)


Sample:

```
        {
            "deviceId": "<<device Id>>",
            "messageId": "<<Client generated unique id for the event>>",
            "eventTs": "1972-01-01T10:00:20.021Z",
            "sentAt": "1972-01-01T10:00:20.021Z",
            "receivedAt": "1970-01-01T00:00:00.021Z",
            "context": {
                // some of the attributes might be null / empty
                "app": {
                    "build": "269999",
                    "version": "30.4"
                },

                //new section added
                "appState": {
                    "inBackground": true, // always false for web
                    "abTestState": "269999"
                },
                 //new section added
                "listeningInfo": {
                    "listenId": "", 
                    "guideId": "", 
                    "parentGuideId":  ""
                },

                "device": {// null for web
                    "manufacturer": "Google",
                    "model": "sdk_gpc_arm64",
                    "name": "e***_a***",
                    "type": "android",
                    "battery_pct": 99  // percentage 0 to 100
                },
                "location": {// null for web
                    "latitude": 34.34,
                    "longitude": 34.34,
                    "city": "<current city of user >",
                    "country": "<current country of user>"
                },
                "page": {
                    "type": ""
                },
                "ip": "<<ip address of device>>", // double check with legal
                "locale": "eg en-US",
                "network": {
                    "bluetooth": false,
                    "carrier": "T-Mobile",
                    "cellular": false,
                    "wifi": true
                },
                "os": {
                    "name": "Android",
                    "version": "12"
                },
                "screen": {
                    "density": 2,
                    "height": 2040,
                    "width": 3840
                },
                "timezone": "<< timezone of the device >>",

                "traits": {
                    "userId": "<<userId of user if known, else null",
                    "deviceId": "<<device Id >>",
                    "isRegistered": false
                },
                "userAgent": "" // added by the clients
            }
        }
```

### Event Specific Attributes

1. `type` - inspired by: https://www.rudderstack.com/docs/event-spec/standard-events/ , https://segment.com/docs/connections/spec/identify/

* `TRACK` - general purpose tracking. Anything from specific actions taken by users, instrumenting a flow like preroll etc.

* `IDENTIFY` - Published when any property associated with a user changes. Eg, when a user logs in / logs out. Braze Id, RevenuecatId created or updated, User updates some profile data, when a logged in user opens a page etc.

* `PAGE` - events are used to track page views within an application. This can provide information on which pages are most popular and how users navigate through the application. 



2. `event` - event name / code
    

3. `properties` - set of attributes specific to the concrete event

NOTE: For Milestone 1 we need to focus on PREROLL related events

NAMING CONVENTION:
{NAMESPACE}_{ACTION}_{ADDITIONAL_INFO}

Namespace: ADS_PREROLL, USER, LISTEN


#### Listen / Station Events

* `USER_PLAY_CLICKED` 
* `LISTEN_SESSION_STARTED`

Sample: 

```
        {
            "type": "TRACK",
            "event": "USER_PLAY_CLICKED",
            "properties": {
                "listenId": "",
                "guideId": "", 
                "parentGuideId":  ""
            }
        }
```

#### Preroll Eligibility Events 

diagram for reference: https://docs.google.com/drawings/d/1yb6W6MRFdBnuq-yqTDt9V46YcS-TwyHECScHUW2XP60/edit

* `ADS_PREROLL_ELIGIBILITY_PLATFORM_REQUESTED` - 'preroll eligibility' - client figures out is user is eligible to play preroll ad. 
* `ADS_PREROLL_ELIGIBILITY_PLATFORM_REQUEST_FAILED` ???
* `ADS_PREROLL_ELIGIBILITY_AD_SERVER_REQUESTED` - 'preroll request' - client requests preroll ad (Final preroll eligibility decision)
Above needs to be clarified with the team. That might be about calling different services to figure out eligibility.
* `ADS_PREROLL_ELIGIBILITY_AD_SERVER_REQUEST_FAILED` ???

Sample:     

```
        {
            "type": "TRACK",
            "event": "PREROLL_ELIGIBILITY_AT_TIME",
            "properties": {
                "listenId": "1234567891234",
                "stationId": "1234567891234",
                "eligibility": true,



                "no_of_prerolls": 1, // not available on that level
    
                // 1. Should that cover single add or multiple adds
                "adds_metadata": {
                    "ad_type": "",
                    "frequency_cap": "???",
                    "adds": [
                        "???"
                    ],
                    "ad_request_id": "",
                    "ad_network_name": "",
                    "ad_format": "",
                    "ad_slot": "",
                    "ad_unit_id": ""
                },


                //From current reporting (preroll eligibility)
                // mostly user settings
                "video": true|false,
                "audio": true|false,
                "ads": true|false,
                "skip": true|false,
                "audioInterval": 123456789,
                "videoInterval": 123456789,
                "assetInterval": 123456789,
                "midroll": true|false,
                "videoDefaulted": true|false,
                "audioDefaulted": true|false,
                "adsDefaulted": true|false,
            }
        }
```




#### ADS_PREROLL Events 

Following events are related to the actual AD content
* `ADS_PREROLL_REQUESTED`
* `ADS_PREROLL_RESPONSE_RECEIVED`
* `ADS_PREROLL_REQUEST_FAILED`

Sample: 

```
        {
            "type": "TRACK",
            "event": "AD_REQUEST",
            "properties": {
                "listenId": "1234567891234",
                "stationId": "1234567891234",
                "eligibility": true,
                "no_of_prerolls": 1,
                "ad_server": "???",

                // optional - used only for type == AD_RESPONSE_FAIL
                "fail_type": "???",
                "error_message": "???",
                "stack_trace": "..."

                "ads_metadata": {
                    "ad_type": "",
                    "frequency_cap": "???",
                    "adds": [
                        "???"
                    ],

                    //to clarify if these are needed - taken from current reporting
                    "ad_request_id": "",
                    "ad_network_name": "",
                    "ad_format": "",
                    "ad_slot": "",
                    "ad_unit_id": "",
                    "elapsed": ""
                },

            }
        }
```

#### AD Playback Events

* `AD_PLAYBACK_STARTED`
* `AD_PLAYBACK_FINISHED`
* `AD_PLAYBACK_FAILED`

Sample: 

```
        {
            "type": "TRACK",
            "event": "AD_PLAYBACK_STARTED",
            "properties": {
                "listenId": "1234567891234",
                "stationId": "1234567891234",
                "no_of_prerolls": 1,
                "current_preroll": 1,

                // optional - used only for type == AD_PLAYBACK_FAILED
                "fail_type": "???",
                "error_message": "???",
                "stack_trace": "..."
            }
        }
```


<!-- // TODO 
1. listenId & stationId - should that be in `app_state` ?
2. add partnerId 
3. properties should support nested objects - e.g. ad_metadata


<!-- do we need more events like;
* `LISTEN_USER_HITS_PAUSE`
* `LISTEN_VOLUME_UP`
* `LISTEN_VOLUME_DOWN`
* `STATION_ADD_TO_FAVORITES`
* `STATION_SHARE`
etc 
? -->

-->


## Resources

* Current TuneIn Report APi: https://tunein.atlassian.net/wiki/spaces/ENG/pages/469206776/Event+Report+API
* Preroll properties - https://tunein.atlassian.net/wiki/spaces/PRODENG/pages/2862186511/Preroll+Eligibility+Reporting
* Current listen reporting - https://tunein.atlassian.net/wiki/spaces/PRODENG/pages/19030128/Tune+Path+Metrics+v2
https://tunein.atlassian.net/wiki/spaces/PRODENG/pages/2796847113/Multiple+Prerolls+and+Midrolls
Current ad reporting API - https://tunein.atlassian.net/wiki/spaces/PRODENG/pages/19029483/Ad+Reporting+Api




// TODOs

1. ad_type should be an enum
2. ELIGIBILITY_REQUEST - only platfrom decides about that, shoudl be single event flow?
    * request to platform
    * final decision on client side (e.g. base on frequency_cap etc)
3. eliminate `eligibility` for `ADS_PREROLL` events  - always `true`
4. `adds_metadata` can be reported only in `ADS_PREROLL`
5. `ad_server` - drop, and keep ad_network_name, ad_format etc
6. `ad_type` - ?? audio/video
7. `frequency_cap` - only valid for eligibility
8. `no_of_prerolls` - inf available later, when preroll is requested
9. playback event - add `ad_type` - video/audio
10. remove `stack_trace` -> `debug_info`, error_type -> excep_type
11. add `ad_unit_id' to playback



https://www.figma.com/file/x5jVXZnZZGDMi8f9pk2FjW/Pre-roll-UI-Analytics-Workflow?node-id=0%3A1&t=rXiR4yIP54kwGQzU-1


Questions:
1. When `received_at` is provided? if that's on the proxy stage - does not make sense have that in individual events
2. 

TODO
1. events to proto
2. chart
3. preroll flow documentation
4. Adding new events / flows - create guide
5. 